# - coding: utf-8 -

import gtk
from gtk import glade
import threading
import exceptions
import gobject
import gconf
import dbus
import dbus.service
import dbus.mainloop.glib
import sys
import os
from pyredmine.pyredmine import PyRedmine 

import subprocess

ID_COL, STATUS_COL, AUTHOR_COL, SUBJECT_COL, \
LASTUPDATE_COL, CATEGORY_COL = range(6)

DATA_DIR = ''

UI_FILE = os.path.join(DATA_DIR, 'data/redmine-desktop-client.ui')
ADD_UI_FILE = os.path.join(DATA_DIR, 'data/create_issue_dialog.ui')

GCONF_PATH = '/apps/redmine-desktop-client'

gobject.threads_init()

def threaded(f):
    def wrapper(*args):
        t = threading.Thread(target=f, args=args)
        t.start()
    return wrapper


class RedmineDesktopClient:
    def __init__(self):

        # Get the widgets
        builder = gtk.Builder()
        if not builder.add_from_file(UI_FILE):
            raise 'Cant load %s' % UI_FILE
        builder.connect_signals(self)
        
        self.main_window = builder.get_object('main_window')
        self.issue_model = builder.get_object('issue_fields_list')
        self.issues_tv = builder.get_object('issues_tv')
        self.actions_menu = builder.get_object('actions_menu')
        self.project_model = builder.get_object('projects_list')
        self.projects_combo = builder.get_object('project_combo')       
        
        self.host_info = builder.get_object('url_lbl')
        self.user_info = builder.get_object('user_lbl')
        self.status_connection = builder.get_object('status_lbl')

        self.about_dialog = builder.get_object('about_dialog')

        self.about_dialog.connect("response", lambda d, r: d.hide())


        ## TODO ESTO SOBRA?!?
        #self.add_issue_window = builder.get_object('add_issue')
        #self.start_date = builder.get_object('entry_start')
        #self.end_date = builder.get_object('entry_end')

        ## dummy entries lines
        self.issue_model.append([12,'issue status', 'author',
                'issue description bla bla bla', 'last update time','issue category'])

        #self.project_model.append(['ueueueu'])

        self.main_window.show()

        self.rmclient = PyRedmine()
        
        self.my_projects = {}
        self.trackers = {}
        self.status = {}
        self.priorities = {}

        #for p in self.rmclient.get_projects():
        #    self.project_model.append([p.name])

        self.host_info.set_text(self.rmclient.projects._site)
        self.user_info.set_text(self.rmclient.projects._user)           
        self.status_connection.set_text('Disconnected')

        self.__get_projects()

        # DEV LINES!
        self.project_model.append(['proyecto'])
        self.my_projects['proyecto'] = str('10278')

    @threaded
    def __get_projects(self):
        try:
            for p in self.rmclient.get_projects():
                self.project_model.append([p.name])
                self.my_projects[str(p.name)] = str(p.id)
                
            self.status_connection.set_text('Connected')
        except Exception, err:
            error = err[0][0][1]
            self.status_connection.set_text('Unable to connect to Redmine!')

#            errDlg = gtk.Dialog('Titulo de la ventana', self.main_window, gtk.MESSAGE_ERROR)
#            ok = errDlg.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
#           
#            response = errDlg.run()
#            if(errDlg.run() == gtk.RESPONSE_OK):
#                errDlg.destroy()


    @threaded
    def __get_project_issues(self, project):
        self.issue_model.clear()
        for i in self.rmclient.get_issues({'project_id' : str(self.my_projects[project])}):
            self.issue_model.append([i.id, i.status, i.author, i.subject,
                 i.updated_on, '']) #, i.category])


    def combobox1_changed_cb(self, data):
        project = self.projects_combo.get_active_text()
        self.__get_project_issues(project)

    def on_button1_clicked(self, data):
        project_name = self.projects_combo.get_active_text()
        self.project = ''
        dlg = IssueDialog(self.rmclient, self.my_projects[project_name])

    def issues_tv_button_press_event_cb(self, data, event):
        if event.button == 3:
            self.actions_menu.popup(None, None, None, event.button,
                    event.time)

    def show_about_dialog(self, data=None, event=None):
        self.about_dialog.show()


    def quit(self, data=None, event=None):
        print '\n... Saliendo!'
        gtk.main_quit()


class IssueDialog:
    def __init__(self, rmclient, project):

        # Get the widgets
        builder = gtk.Builder()
        if not builder.add_from_file(ADD_UI_FILE):
            raise 'Cant load %s' % ADD_UI_FILE
        builder.connect_signals(self)

        self.add_issue_window = builder.get_object('add_issue')
        self.subject = builder.get_object('entry_subject')
        self.description = builder.get_object('text_description')
        self.tracker = builder.get_object('combobox_tracker')
        self.status = builder.get_object('combobox_status')
        self.priority = builder.get_object('combobox_priority')
        self.assigned = builder.get_object('combobox_assigned')
        self.start_date = builder.get_object('entry_start')
        self.end_date = builder.get_object('entry_end')        
        self.estimated = builder.get_object('entry_estimated')
        self.done = builder.get_object('combobox_done')

        ## FEATURE: USAR ESTA MISMA CLASE PARA ACTUALIZAR PETICION
        ## se detecta si viene una peticion, todas las extensiones
        ## de la ui iria a un metodo y/o clase aparte
        ## Tambien habria que rellenar los datos de la peticion
        ## en sus respectivos lugares
        #
        #if (issue):
        #    self.vbox = builder.get_object('vbox_add_issue')
        #
        #    # aqui debemos iterar los journals
        #    for i in [0,1]: 
        #        _issue_journals_builder = gtk.Builder()
        #        _issue_jounals_builder.add_from_file('data/issue_journals.ui')
        #
        #        # y vamos añadiendo
        #        self.vbox.add(nu_bild.get_object('vbox1'))
        #
        #
        # # tras iterar, añade lo necesario para añadir comentario
        #
        # 

        self.add_issue_window.show()
      
        self.attributes = {}
        self.attributes['project_id'] = project

        self.rmclient = rmclient
    
    def on_start_date_btn_clicked(self, data):
        self.start_date.set_text(self.get_date_from_dialog())

    def on_end_date_btn_clicked(self, data):
        self.end_date.set_text(self.get_date_from_dialog())

    def get_date_from_dialog(self):
        ## NASTY CALENDAR DIALOG, MAKE THIS USING GTK!?!
        calendar_dlg = subprocess.Popen(['/usr/bin/zenity','--calendar'], stdout = subprocess.PIPE)
        date_str = calendar_dlg.stdout.read()
        date_str = date_str.splitlines()[0]
        return date_str
        

    def on_ok_btn_released(self, data):
        self.attributes = self.get_attributes() 

        print '\n', self.attributes, '\n'

        try:
            self.rmclient.create_issue(self.attributes)
        except:
            print 'No se pudo crear'

    def on_cancel_btn_released(self, data):
        print 'Add issue Cancelled!\n'
        self.add_issue_window.destroy()


    def get_attributes(self):

        self.attributes['subject'] = self.subject.get_text()

        buffer = self.description.get_buffer()
        self.desc = buffer.get_text(buffer.get_start_iter(), buffer.get_end_iter())        
         
        self.attributes['description'] = self.desc
        self.attributes['tracker'] = self.tracker.get_active_text()
        self.attributes['status'] = self.status.get_active_text()
        self.attributes['priority'] = self.priority.get_active_text()
        self.attributes['assigned'] = self.assigned.get_active_text()
        self.attributes['start_date'] = self.start_date.get_text()
        self.attributes['end_date'] = self.end_date.get_text()
        self.attributes['estimated'] = self.estimated.get_text()
        self.attributes['done'] = self.done.get_active_text() 

        return self.attributes


if __name__ == '__main__':
    import os
    import sys

    rmdclient = RedmineDesktopClient()
    gtk.main()

